from __future__ import annotations

from typing import TYPE_CHECKING
from typing import final

from .abc import TelegramMethod

if TYPE_CHECKING:
    from yatbaf.typing import NoneBool

# from datetime import timedelta


@final
class BanChatMember(TelegramMethod[bool]):
    """See :meth:`yatbaf.bot.Bot.ban_chat_member`"""

    chat_id: str | int
    user_id: int
    until_date: int | None = None  # TODO: timedelta
    revoke_messages: NoneBool = None
